{$S40}
PROGRAM JumpSTART_2;

{
   by Paul R. Lefebvre      (c) 1991,1992

   VERSION 2.1   March 3, 1992
     - Allows JumpSTART to be an installed application
     - Allows instant loading of JMP files
     - NeoDesk 3 icon added to LZH

   VERSION 2.2   March 8, 1992
     - Allows instant loading of JMP files from any drive, not just the
       JumpSTART directory
     - Clicking JumpSTART 2 title box brings up address information
     - Improved directory handling so that JumpSTART remembers the last
       directory accessed when ADDing files
     
   JumpSTART provides the user with a quick and easy way to start
   commonly used programs.  All of your programs can be assigned a 
   20 character description and can be started with a single mouse
   click.

   Up to 63 file descriptions can be stored in one file list.  New
   file lists can be loaded at any time.
   
   Works in Low, Medium, and High resolutions.
  
}
  
{$I a:\source\jumpst2.i}
{$I f:\gemsubs.pas}
{$I f:\auxsubs.pas}
{$I f:\gemdos.pas}
{$I f:\xbios.pas}

CONST
   MAX_LIST_SIZE = 63;
   MAX_ONSCREEN  = 21;
   BLANKLINE = '                    '; { 20 spaces }
      
TYPE
   record_type                    = RECORD
                                       description : STRING[80];
                                       file_path    : Str255;
                                    END;
   record_array                   = ARRAY[1..max_list_size] OF record_type;
   ext_type                       = STRING[3];
                    
VAR
   old_colors                                  : ARRAY[0..15] OF INTEGER;
   load_dir, title, jump_filename, autoload_fn : Str255;
   main_dg_ptr                                 : Dialog_Ptr;
   first, last, list_size, ivoid, pushed,
   load_drv, rez, xres, yres, first_file,
   last_file, pages, current_page, main_dialog : INTEGER;
   keystate                                    : BYTE;
   changes, redraw, bvoid                      : BOOLEAN;
   file_list                                   : record_array;
   c_load_dir                                    : C_String;
      
{ Temporary variables : }
   lp : INTEGER;
      
{ ===================================================================== }

PROCEDURE chdir( path : Str255; drv : INTEGER );

{ Changes the current drive and directory. }

VAR
  c_path : C_String;
  ivoid : INTEGER;
BEGIN
   P_To_Cstr( path, c_path );
   ivoid := dsetdrv( drv );
   ivoid := dsetpath( c_path )
END; { chdir }

{ --------------------------------------------------------------------- }

PROCEDURE init_screen;

{ Clears the screen and sets the mouse form to an arrow. 
  Gets the screen resolution. }

VAR
   x, y, w, h : INTEGER;
   
BEGIN
   Clear_Screen;
   CASE rez OF
      0 : x := 15;
      1 : x := 3;
      2 : x := 1
   END;
   FOR y := 0 TO x DO
      w := setcolor( y, old_colors[y] );
   Work_Rect( 0, x, y, w, h );
   Paint_Outline( FALSE );
   Paint_Color( 1 );
   IF yres = 2 THEN
      Paint_Color( 3 );
   Paint_Style( 5 );
   IF yres = 2 THEN
      Paint_Style( 1 );
   Paint_Rect( x, y, w, h );
   Init_Mouse
{   chdir( load_dir, load_drv )}
END; { init_screen }

{ ---------------------------------------------------------------------- }

PROCEDURE set_up_everything;

{ Program initialization. }

VAR
   lp : INTEGER;

BEGIN
   init_screen;
   Find_Dialog( main_dialog, main_dg_ptr );
   Center_Dialog( main_dg_ptr );

END; { set_up_everything }

{ --------------------------------------------------------------------- }

PROCEDURE show_help_dialog;

{ Put the help dialog box on the screen }

VAR
   void : INTEGER;
   help_dg_ptr : Dialog_Ptr;

BEGIN
   Find_Dialog( HELP_DG,help_dg_ptr );
   Center_Dialog( help_dg_ptr );
   void := Do_Dialog( help_dg_ptr,0 );
   Obj_SetState( help_dg_ptr, void, Normal, TRUE )
END; { show_help_dialog }

{ --------------------------------------------------------------------- }

PROCEDURE change_title( VAR title : Str255; change : BOOLEAN );

{ This procedure allows the user to enter a new title if 'change' is TRUE.
  If 'change' is FALSE, it just puts the title in the dialog box. }

VAR
   title_dg_ptr : Dialog_Ptr;
   clicked : INTEGER;
   quit_str : Str255;
     
BEGIN
   IF change THEN BEGIN
      Find_Dialog( TITLE_DG, title_dg_ptr );
      Set_DText( title_dg_ptr, TITLE_ED, title, System_Font, TE_Left );   
      Center_Dialog( title_dg_ptr );
      clicked := Do_Dialog( title_dg_ptr, TITLE_ED );
      IF ( clicked = OK_TITLE ) THEN BEGIN
         Get_DEdit( title_dg_ptr, TITLE_ED, title );
         changes := TRUE;
         change_title( title, FALSE )
      END;
      Obj_SetState( title_dg_ptr, clicked, Normal, FALSE );
      redraw := TRUE
   END
   ELSE BEGIN
      IF rez = 0 THEN
         IF length( title ) > 18 THEN
            title := copy( title, 1, 18 );
      Set_DText( main_dg_ptr, JUMPNAME, title, System_Font, TE_Center );
      redraw := FALSE
   END
END; { change_title }

{ --------------------------------------------------------------------- }

PROCEDURE bubble_sort( VAR file_list : record_array; list_size : INTEGER );

{  Sorts the file_list record.  Since it contains at most 62 entries, a 
   bubble sort is fast enough. }

VAR
   sorted : BOOLEAN;
   i : INTEGER;
   temp : record_type;
      
BEGIN
   REPEAT
      sorted := TRUE;
      FOR i := 1 TO ( list_size-1 ) DO
         IF file_list[i].description > file_list[i+1].description THEN BEGIN
            temp := file_list[i];
            file_list[i] := file_list[i+1];
            file_list[i+1] := temp;
            sorted := FALSE
         END
   UNTIL sorted
END; { bubble_sort }

{ ---------------------------------------------------------------------- }

PROCEDURE show_files( VAR file_list : record_array; first_file, list_size,
                      current_page, pages : INTEGER );

{ Puts the list of files into the main dialog box. }

VAR
   lp : INTEGER;
   current_page_str : Str255;
   
BEGIN
   bubble_sort( file_list, list_size );
   FOR lp := 0 TO ( MAX_ONSCREEN-1 ) DO
      Set_DText( main_dg_ptr, ( NAME_1+lp ), file_list[lp+first_file].description,
                 System_Font, TE_Center );
   current_page_str := 'Page 1 of 1';
   current_page_str[6] := chr( current_page+48 );
   current_page_str[11] := chr( pages+48 );
   Set_DText( main_dg_ptr, PAGE_NO, current_page_str, System_Font, TE_Center )

END; { show_files }

{ --------------------------------------------------------------------- }

PROCEDURE set_up_files( VAR first_file, pages, current_page : INTEGER;
                        list_size : INTEGER );

{ Determines how many pages of files there are.  Maximum of 3.  }

BEGIN
   first_file := 1;
   pages := 3;
   IF list_size <= 42 THEN
      pages := 2;
   IF list_size <= 21 THEN
      pages := 1;
   current_page := 1
END; { set_up_files }

{ --------------------------------------------------------------------- }

PROCEDURE page_updown( VAR first_file, current_page : INTEGER;
                       pages : INTEGER; up : BOOLEAN );

{ Determines which page is to be displayed. }

VAR
   current_page_str : Str255;

BEGIN
   redraw := FALSE;
   IF ( pages <> 1 ) THEN BEGIN
      IF up THEN BEGIN
         IF ( pages - current_page ) >= 1 THEN BEGIN
            current_page := current_page + 1;
            redraw := TRUE
         END;
      END
      ELSE
         IF current_page <> 1 THEN BEGIN
            current_page := current_page - 1;
            redraw := TRUE
         END;
      first_file := (MAX_ONSCREEN * (current_page-1))+1
   END
END; { page_updown }

{ --------------------------------------------------------------------- }
PROCEDURE enable_radio_buttons;

BEGIN
   Obj_SetState( main_dg_ptr, RUN_BUT, Selected, FALSE );
   Obj_SetState( main_dg_ptr, PATH_BUT, Normal, FALSE );
   Obj_SetState( main_dg_ptr, DESC_BUT, Normal, FALSE );
   Obj_SetState( main_dg_ptr, REMOVE, Normal, FALSE )
END;

{ --------------------------------------------------------------------- }

PROCEDURE load_jmp_file( jump_filename : Str255; VAR title : Str255;
                         VAR file_list : record_array;
                         VAR list_size, first_file, pages,
                             current_page : INTEGER );

{ This procedure loads a JMP file from the disk into the corresponding
  record array.  }
  
VAR
   jump_file : TEXT;
   temp_title : Str255;
   dummy_line : STRING[80];
   error, eq, len, count : INTEGER;
     
BEGIN
   IO_Check( FALSE );
   reset( jump_file, jump_filename );
   error := IO_Result;
   IF ( error = NO_ERROR ) THEN BEGIN
      Set_Mouse( M_Bee );
      readln( jump_file, temp_title );
      count := 0;
      WHILE ( NOT eof( jump_file ) ) AND ( count < MAX_LIST_SIZE ) DO BEGIN
         readln( jump_file, dummy_line );
         IF ( dummy_line <> '@END' ) AND ( dummy_line <> ';' ) AND
            ( dummy_line <> '' ) AND ( pos( '=', dummy_line ) <> 0 )THEN BEGIN
            { Only add line if it is not empty or a comment or blank }   
            count := count + 1;
            eq := pos( '=', dummy_line );
            len := length( dummy_line );
            file_list[count].description := copy( dummy_line, 1, (eq - 1) );
            file_list[count].file_path := copy( dummy_line, (eq + 1), len )
         END
       END;
       close( jump_file );
       Set_Mouse( M_Arrow );
       IF count <> 0 THEN BEGIN
          changes := FALSE;
          list_size := count;
          FOR eq := (list_size+1) TO MAX_LIST_SIZE DO BEGIN
             file_list[eq].description := BLANKLINE;
             file_list[eq].file_path := '';
          END;
          title := temp_title;
          change_title( title, FALSE);
          set_up_files( first_file, pages, current_page, list_size );
          show_files( file_list, first_file, list_size, current_page, pages );
          enable_radio_buttons
       END
       ELSE
          eq := Do_Alert( '[1][Invalid file format.][ Ok ]', 1 );
    END
    ELSE
       error := Do_Alert( '[1][.JMP file not found.][ Ok ]',1)
END; { load_jump_file }

{ ----------------------------------------------------------------------- }

FUNCTION get_filename( ext : ext_type; VAR filename : Str255 ) : BOOLEAN;

{ Puts a file selector on screen with the current drive and directory.
  Returns TRUE if a OK is selected, FALSE if cancel is selected. }

VAR
   c_def_path : C_String;
   def_path : Str255;
   drive : INTEGER;

BEGIN
   drive := dgetdrv;
   ivoid := dgetpath( c_def_path, drive+1 );
   C_To_Pstr( c_def_path, def_path );
   def_path := concat( chr(drive+65),':', def_path,'\*.',ext );
   get_filename := Get_In_File( def_path, filename )
END; { get_filename }

{ --------------------------------------------------------------------- }

PROCEDURE parse( VAR path, filename : Str255 );

{ Takes a complete pathname and separates the filename and the directory. }

VAR
   bs : INTEGER;
   temp1, temp2 : Str255;
      
BEGIN
   temp2 := '';
   REPEAT
      bs := pos( '\', path );
      IF ( bs <> 0 ) THEN BEGIN
         temp1 := copy( path, 1, bs );
         temp2 := concat( temp2, temp1 );
         delete( path, 1, bs )
      END
   UNTIL ( bs = 0 );
   filename := path;
   path := temp2
END; { parse }

{ ---------------------------------------------------------------------- }

PROCEDURE get_extender( filename : Str255; VAR ext : ext_type );

{  Gets the extender from a TOS filename. }

VAR
   dot : INTEGER;
   temp : Str255;

BEGIN
   temp := filename;
   parse( temp, filename );
   dot := pos( '.', filename );
   IF ( dot = 0 ) THEN
      ext := ''
   ELSE
      ext := copy( filename, (dot+1), length( filename )-dot )
END;

{ --------------------------------------------------------------------- }

PROCEDURE save_file( jump_filename : Str255; file_list : record_array;
                     title : Str255; list_size : INTEGER );

{ Saves the file_list to disk.  Prompts for a filename. }
VAR
   temp : Str255;
   jump_file : TEXT;
   count : INTEGER;
   ext : ext_type;
   
BEGIN
   redraw := TRUE;
   IF changes THEN BEGIN
      temp := jump_filename;
      parse( temp, jump_filename );
      IF get_filename( 'JMP', jump_filename ) THEN BEGIN
         get_extender( jump_filename, ext );
         IF ext = '' THEN
            jump_filename := concat( jump_filename, '.JMP');
         rewrite( jump_file, jump_filename );
         Set_Mouse( M_Bee );
         writeln( jump_file, title );
         count := 0;
         REPEAT
            count := count + 1;
            write( jump_file, file_list[count].description );
            write( jump_file, '=' );
            writeln( jump_file, file_list[count].file_path )
         UNTIL count = list_size;
         close( jump_file );
         Set_Mouse( M_Arrow )
      END
   END
END;
{ --------------------------------------------------------------------- }

PROCEDURE change_description( VAR file_list : record_array;
                              selected_file, first_file : INTEGER );

{  Change the description for a filename. }

VAR
   desc_dg_ptr : Dialog_Ptr;
   spc, bs, clicked : INTEGER;
   temp, filename : Str255;

BEGIN
   Find_Dialog( DESC_DG, desc_dg_ptr );
   temp := file_list[selected_file].file_path;
   spc := pos( ' ', temp );  { Check if there is a parameter }
   IF ( spc <> 0 )THEN          { There is, so show it, not the executable file }
      temp := copy( temp, (spc+1), length(temp) );
   parse( temp, filename );
   Set_DText( desc_dg_ptr, PATH_DSC, filename, System_Font, TE_Left );
   temp := file_list[selected_file].description;
   IF ( temp = BLANKLINE ) THEN
      temp := '';
   Set_DText( desc_dg_ptr, FILE_DSC, temp, System_Font, TE_Left );
   Center_Dialog( desc_dg_ptr );
   clicked := Do_Dialog( desc_dg_ptr, FILE_DSC );
   IF ( clicked = OK_DESC ) THEN BEGIN
      changes := TRUE;
      Get_DEdit( desc_dg_ptr, FILE_DSC, temp );
      IF ( temp = '' ) THEN
         temp := filename;
      file_list[selected_file].description := temp
   END
   ELSE
      IF ( temp = '' ) THEN
         file_list[selected_file].description := filename;
   Obj_SetState( desc_dg_ptr, clicked , Normal, FALSE );
   redraw := TRUE;
END; { change_description }

{ ---------------------------------------------------------------------- }

PROCEDURE remove_file( VAR file_list : record_array; VAR list_size,
                       pages, first_file, current_page : INTEGER;
                       erased : INTEGER );

{  Removes a filename and adjust the file_list. }

VAR
   i : INTEGER;

BEGIN
   changes := TRUE;
   FOR i := erased TO ( list_size-1 ) DO
      file_list[i] := file_list[i+1];
   file_list[list_size].description := blankline;
   file_list[list_size].file_path := '' ;
   list_size := list_size-1;
   IF list_size = MAX_LIST_SIZE-1 THEN
      Obj_SetState( main_dg_ptr, ADD_BUT, Disabled, FALSE );
   IF ( list_size = MAX_ONSCREEN ) THEN BEGIN
      pages := 1;
      set_up_files( first_file, pages, current_page, list_size )
   END;
   IF ( list_size = MAX_ONSCREEN*2 ) THEN BEGIN
      pages := 2;
      IF ( current_page > pages ) THEN BEGIN
         current_page := 2;
         first_file := ( MAX_ONSCREEN * (current_page-1))+1
      END
   END;
   redraw := TRUE

END; { remove_file }

{ --------------------------------------------------------------------- }

FUNCTION change_filepath( VAR file_list : record_array;
                           selected_file : INTEGER) : BOOLEAN;

{  Displays a file selector to allow the user to select a new filepath. }

VAR

   c_def_path : C_String;
   def_path, temp, filename, parameter : Str255;
   drive, spc : INTEGER;
   ext : ext_type;
   bvoid : BOOLEAN;
         
BEGIN
   change_filepath := FALSE;
   temp := file_list[selected_file].file_path;

   IF ( temp <> '') THEN BEGIN
      spc := pos( ' ', temp );
      IF ( spc <> 0 ) THEN
         temp := copy( temp, (spc+1), length(temp) );
      parse( temp, filename );
      temp := concat( temp, '*.*' );
      IF filename = '*.*' THEN
         filename := '';
   END
   ELSE BEGIN
      drive := dgetdrv;
      ivoid := dgetpath( c_def_path, drive+1 );
      C_To_Pstr( c_def_path, temp );
      temp := concat( chr(drive+65),':', temp,'\*.*' );
      filename := ''
   END;
   bvoid := Get_In_File( temp, filename );
   IF bvoid AND ( filename <> '' ) THEN BEGIN
      get_extender( filename, ext );
      IF ( ext = 'PRG' ) OR ( ext = 'APP' ) OR ( ext = 'TOS' ) OR
         ( ext = 'TTP' ) OR ( ext = 'JMP' ) THEN BEGIN
         file_list[selected_file].file_path := filename;
         drive := ord( filename[1] )-65;
         parse( filename, temp );
         chdir( filename, drive );
         change_filepath := TRUE;
         changes := TRUE
      END
      ELSE BEGIN
         parameter := filename;
         temp := filename;
         parse( temp, filename );
         temp := concat( temp, '*.*' );
         bvoid := Get_In_File( temp, filename );
         IF bvoid AND ( filename <> '' ) THEN BEGIN
            get_extender( filename, ext );
            IF ( ext = 'PRG' ) OR ( ext = 'APP' ) OR
               ( ext = 'TOS' ) OR ( ext = 'TTP' ) THEN BEGIN
               file_list[selected_file].file_path :=
                             concat( filename, ' ',parameter );
               drive := ord( parameter[1] )-65;
               parse( parameter, filename );
               chdir( parameter, drive );
               change_filepath := TRUE;
               changes := TRUE
            END
         END
      END
   END
END;
{ --------------------------------------------------------------------- }

PROCEDURE show_filepath( VAR file_list : record_array;
                           selected_file : INTEGER );

{  Displays a dialog box showing the complete filepath. }

VAR
   path_dg_ptr : Dialog_Ptr;
   temp : Str255;
   spc, clicked : INTEGER;
   bvoid : BOOLEAN;
                                 
BEGIN
   Find_Dialog( PATH_DG, path_dg_ptr );
   temp := file_list[selected_file].file_path;
   spc := pos( ' ', temp );  { Check if there is a parameter }
   IF ( spc <> 0 )THEN          { There is, so show it, not the executable file }
      temp := copy( temp, (spc+1), length(temp) );

   Set_DText( path_dg_ptr, PATHTEXT, temp, System_Font, TE_Center );
   Set_DText( path_dg_ptr, SHOWDESC, file_list[selected_file].description,
              System_Font, TE_Center );
   Center_Dialog( path_dg_ptr );

   clicked := Do_Dialog( path_dg_ptr, 0 );
   IF ( clicked = CHG_PATH ) THEN
      bvoid := change_filepath( file_list, selected_file );
   Obj_SetState( path_dg_ptr, clicked, Normal, FALSE );
   redraw := TRUE
END; { change_filepath }
                                 
{ ---------------------------------------------------------------------- }

PROCEDURE add_file( VAR file_list : record_array;
                    VAR list_size, pages : INTEGER;
                    current_page, first_file : INTEGER );

{ Add file control routine.  Calls change_filepath and change_description. }

VAR
   return : BOOLEAN;
   current_page_str : Str255;
                       
BEGIN
   REPEAT
      return := change_filepath( file_list, list_size+1 );
      IF return THEN BEGIN
         change_description( file_list, list_size+1, first_file );
         list_size := list_size + 1;
         IF list_size = 1 THEN
            enable_radio_buttons;
         IF ( list_size = MAX_ONSCREEN+1 ) OR 
            ( list_size = MAX_ONSCREEN*2+1 ) THEN
               pages := pages + 1;
         show_files( file_list, first_file, list_size, current_page, pages );
         changes := TRUE;
         IF list_size = MAX_LIST_SIZE THEN BEGIN
            Obj_SetState( main_dg_ptr, ADD_BUT, Disabled, FALSE );
            return := FALSE
         END
      END
   UNTIL NOT return
END;
      
{ ---------------------------------------------------------------------- }

PROCEDURE check_radio_buttons( VAR file_list : record_array;
                               VAR selected_file, list_size, first_file,
                                   current_page, pages : INTEGER );

{ When a filename is clicked on this procedure determines what to do.
  1.) Run the file
  2.) New Description
  3.) New filepath
  4.) Remove file  }
  
VAR
   runpath, filepath, filename : Str255;
   spc, drv, file_number : INTEGER;
   ext : ext_type;
       
BEGIN
   redraw := FALSE;
   file_number := selected_file-NAME_1+first_file;
   IF ( Obj_State( main_dg_ptr, RUN_BUT ) & Selected <> 0 ) THEN
      IF ( file_number <= list_size ) THEN BEGIN
         filepath :=  file_list[file_number].file_path;
         spc := pos( ' ', filepath );
         IF ( spc <> 0 ) THEN
            runpath := copy( filepath, (spc+1), length( filepath ) )
         ELSE
            runpath := filepath;
         drv := ord( runpath[1] )-65;
         parse( runpath, filename );
         get_extender( filename, ext );
         IF ( ext = 'JMP' ) THEN BEGIN
            load_jmp_file( filepath, title, file_list, list_size,
                           first_file, pages, current_page );
            show_files( file_list, first_file, list_size, current_page, pages );
            redraw := TRUE
         END
         ELSE BEGIN
            chdir( runpath, drv );
            End_Dialog( main_dg_ptr );
            Draw_String( (240 DIV xres)-length( file_list[file_number].description ) DIV 2,
                         15 DIV yres, file_list[file_number].file_path );
            Set_Mouse( M_Bee );
            Exit_Gem;
            IF spc <> 0 THEN
               chain( filepath )
            ELSE
               chain( filename );
            ivoid := Init_Gem;
            IF ivoid < 0 THEN
               Halt;
            init_screen;
            redraw := TRUE
         END
      END;
   IF ( Obj_State( main_dg_ptr, DESC_BUT ) & Selected <> 0 ) THEN
      IF ( file_number <= list_size ) THEN BEGIN
         change_description( file_list, file_number, first_file );
         show_files( file_list, first_file, list_size, current_page, pages )
      END;
   
   IF ( Obj_State( main_dg_ptr, REMOVE ) & Selected <> 0 ) THEN
      IF ( file_number <= list_size ) THEN BEGIN
         remove_file( file_list, list_size, pages, first_file,
                      current_page, file_number );
         show_files( file_list, first_file, list_size, current_page, pages )
      END;

   IF ( Obj_State( main_dg_ptr, PATH_BUT ) & Selected <> 0 ) THEN
      IF ( file_number <= list_size ) THEN
         show_filepath( file_list, file_number ) 
         
END; { check_radio_buttons }

{ -------------------------------------------------------------------- }

PROCEDURE use_da;

{  Puts a menu bar on the screen to allow the user to access the Desk
   accessories. }

VAR
   da_menu : Menu_Ptr;
   dummy, which : INTEGER;
   msg : Message_Buffer;
   
BEGIN
   End_Dialog( main_dg_ptr );
   redraw := TRUE;
   da_menu := New_Menu( 9, ' Return to JumpSTART' );
   Draw_Menu( da_menu );
   REPEAT
      which := Get_Event( E_Message, 0, 0, 0, 0,
               FALSE, 0, 0, 0, 0, FALSE, 0, 0, 0, 0, msg,
               dummy, dummy, dummy, dummy, dummy, dummy )
   UNTIL msg[3] = Desk_Title;
   Menu_Normal( da_menu, msg[3] );
   Erase_Menu( da_menu );
   Delete_Menu( da_menu );
   init_screen
END;

{ ===================================================================== }
   
BEGIN { main }
   IF (Init_Gem >= 0) THEN BEGIN
      rez := getrez;
      CASE rez OF
         0 : BEGIN
		jump_filename := 'STARTLOW.JMP';
                xres := 2;
                yres := 2;
                main_dialog := MAIN_DG2;
                FOR lp := 0 TO 15 DO
                   old_colors[lp] := setcolor( lp, -1 )
             END;
         1 : BEGIN
         	jump_filename := 'STARTMED.JMP';
                xres := 1;
                yres := 2;
                main_dialog := MAIN_DG;
                FOR lp := 0 TO 3 DO
                   old_colors[lp] := setcolor( lp, -1 )
             END;
         2 : BEGIN
         	jump_filename := 'STARTHI.JMP';
                xres := 1;
                yres := 1;
                main_dialog := MAIN_DG;
                FOR lp := 0 TO 1 DO
                   old_colors[lp] := setcolor( lp, -1 )
             END
      END;
      autoload_fn := concat( 'A:\', jump_filename);
      IF (cmd_args > 0) THEN
         cmd_getarg( 1, jump_filename );
      IF Load_Resource( 'jumpst2.rsc' ) THEN BEGIN
         load_drv := dgetdrv;
         ivoid := dgetpath( c_load_dir, load_drv+1 );
         C_To_Pstr( c_load_dir, load_dir );
         load_dir := concat( chr( load_drv+65 ), ':', load_dir, '\' );
         changes := FALSE;
   
         set_up_everything;
         FOR lp := 0 TO (NAME_21-NAME_1) DO BEGIN
            file_list[lp+1].description := BLANKLINE;
            file_list[lp+1].file_path := '';
            Set_DText( main_dg_ptr, NAME_1+lp, BLANKLINE, System_Font, TE_Center );
            Obj_SetState( main_dg_ptr, NAME_1+lp, Normal, FALSE )
         END;
         list_size := 0;
         
         set_up_files( first_file, pages, current_page, list_size );
         load_jmp_file( jump_filename, title, file_list, list_size,
                        first_file, pages, current_page );
         redraw := TRUE;

         REPEAT
            IF redraw THEN
               pushed := Do_Dialog( main_dg_ptr, 0 ) & $7FFF
            ELSE
               pushed := Redo_Dialog( main_dg_ptr, 0 ) & $7FFF;
               
            IF ( pushed >= NAME_1 ) AND ( pushed <= NAME_21 ) THEN
               check_radio_buttons( file_list, pushed, list_size, first_file,
                                    current_page, pages )
            ELSE
               CASE pushed OF
                  JUMPNAME : change_title( title, TRUE );

                  UP_BUT   : BEGIN
                                page_updown( first_file, current_page, pages,
                                             TRUE );
                                show_files( file_list, first_file, list_size,
                                            current_page, pages )
                             END;
 
                  DOWN_BUT : BEGIN 
                                page_updown ( first_file, current_page, pages,
                                              FALSE );
                                show_files( file_list, first_file, list_size,
                                            current_page, pages )
                             END;

                  HELP_BUT : BEGIN
                                show_help_dialog;
                                redraw := TRUE
                             END;

                  LOAD_BUT : IF get_filename( 'JMP', jump_filename ) THEN BEGIN
                                load_jmp_file( jump_filename, title, file_list,
                                               list_size, first_file, pages,
                                               current_page );
                                redraw := TRUE
                             END;
                             
                  SAVE_BUT : save_file( jump_filename, file_list, title,
                                        list_size );
                                        
                  READ_A   : BEGIN
                                load_jmp_file( autoload_fn, title, file_list,
                                               list_size, first_file, pages,
                                               current_page );
                                redraw := TRUE
                             END;
                                            
                  ADD_BUT  : add_file( file_list, list_size, pages,
                                       current_page, first_file );
                  
                  DA_BUT   : use_da;
                  
                  TITLEBOX : BEGIN
                                ivoid := 
Do_Alert( '[0][ GEnie : P.LEFEBVRE  |Mail : 94 Bailey Ave | Lewiston, ME 04240][ OK ]',1);
                                redraw := FALSE
                             END;

                  OTHERWISE : redraw := FALSE

               END; { CASE, ELSE }
            Obj_SetState( main_dg_ptr, pushed, Normal, NOT redraw );
            IF changes THEN
               Obj_SetState( main_dg_ptr, SAVE_BUT, Normal, FALSE );
         
         UNTIL pushed = QUIT_BUT
      END
      ELSE
         ivoid := Do_Alert( '[3][Can''t find JUMPST2.RSC!][ Abort ]', 1 );
      Exit_Gem
   END
END. { main }
